#include "RadiographyImage.h"
#include "MyMatrix.h"
#include <iostream>
#include <qt3/qimage.h>
// Constructor
RadiographyImage::RadiographyImage(const string& img_str, unsigned int size, bool inverted) 
	:_imgStr(img_str) {

  // create Image object
  //(this->_imageData) = ImageType::New();
  
  // read image data
  readImage(size, inverted);
 
  cerr << "created" << endl; 

  
}

void
RadiographyImage::readImage(unsigned int im_size, bool inverted) {
  // set error variable to 0
  errno = 0;
  int numElements = im_size*im_size;
  
  // open Image data file for reading
  FILE* im_fid = fopen((this->_imgStr).c_str(), "r");
  if (errno) {
    perror("Invalid data file name");
    errno = 0;
  }
  
  // allocate memory for image data
  unsigned short( * const image_data)[im_size] = new unsigned short[im_size][im_size];
  
  // read file data and store in image data buffer
  int num_read = 0;
  
  int xpos=0, ypos=0;
  unsigned short buf;
  unsigned short cutter = 0x0fff; // to cut the highest 2 invalid bits
  for (int count=0; count<numElements; count++) {
    //calculate indizes of matrix position
    ypos = (int) fmod((double) count, (double) im_size);
    xpos = count / im_size;

    // fread (buffer, size_of_elem_in_bytes, num_elems, stream)
    num_read = fread(&buf,  2, 1, im_fid);
    if (num_read != 1) {
    cerr << "ERROR: wrong number of data elements read" << endl;
    }

    // only 14 of 16 bit are valid; cut the highest two
    unsigned short cutted_val = buf & cutter;
    // store in data buffer; invert if necessary
    if (inverted) image_data[xpos][ypos] = cutter - cutted_val;
    else image_data[xpos][ypos] = cutted_val;
    
    //cerr << "index: " << count << " read: " << image_data[xpos][ypos] << endl;
  }
  
  // create data Matrix
  this->_imageData = new MyMatrix<unsigned short>((DataType **) image_data, im_size, im_size);
  
}

